import {
  Button,
  Card,
  Col,
  DatePicker,
  Divider,
  Dropdown,
  Form,
  Icon,
  Input,
  InputNumber,
  Popconfirm,
  Row,
  Select,
  message,
} from 'antd';
import React, { Component, Fragment } from 'react';

import { Dispatch, Action } from 'redux';
import { FormComponentProps } from 'antd/es/form';
import { PageHeaderWrapper } from '@ant-design/pro-layout';
import { SorterResult } from 'antd/es/table';
import { connect } from 'dva';
import { StateType } from './model';
import CreateForm from './components/CreateForm';
import EditForm from './components/EditForm';
import StandardTable, { StandardTableColumnProps } from './components/StandardTable';
import UpdateForm, { FormValueType } from './components/UpdateForm';
import { TableListItem, TableListPagination, TableListParams } from './data.d';

import styles from './style.less';

const FormItem = Form.Item;
const { Option } = Select;
const getValue = (obj: { [x: string]: string[] }) =>
  Object.keys(obj)
    .map(key => obj[key])
    .join(',');

// type IStatusMapType = 'default' | 'processing' | 'success' | 'error';
// const statusMap = ['default', 'processing', 'success', 'error'];
// const status = ['关闭', '运行中', '已上线', '异常'];

interface TableListProps extends FormComponentProps {
  dispatch: Dispatch<Action<'area/add' | 'area/fetch' | 'area/remove' | 'area/update'>>;
  loading: boolean;
  area: StateType;
}

interface TableListState {
  modalVisible: boolean;
  updateModalVisible: boolean;
  expandForm: boolean;
  selectedRows: TableListItem[];
  formValues: { [key: string]: string };
  stepFormValues: Partial<TableListItem>;
  cityCode: string;
  // options: Iteam[];
}

/* eslint react/no-multi-comp:0 */
@connect(
  ({
    area,
    loading,
  }: {
    area: StateType;
    loading: {
      models: {
        [key: string]: boolean;
      };
    };
  }) => ({
    area,
    loading: loading.models.area,
  }),
)
class TableList extends Component<TableListProps, TableListState> {
  /**
   * 只保存交互用变量，其他在模型中处理
   */
  state: TableListState = {
    modalVisible: false,
    updateModalVisible: false,
    expandForm: false,
    selectedRows: [],
    formValues: {},
    stepFormValues: {},
    cityCode: ""
  };

  columns: StandardTableColumnProps[] = [
    {
      title: '代码',
      dataIndex: 'area_code',
      key: 'id',
    },
    {
      title: '地区',
      dataIndex: 'area_name',
      key: 'name',
    },
    {
      title: '操作',
      render: (text, record) => (
        // console.log(record.area_code.slice(4, 6) ),
        record.area_code.slice(4, 6) !== "00" ?
          <Fragment>
            <a onClick={() => this.handleUpdateModalVisible(true, record)}>修改</a>
            <Divider type="vertical" />
            <Popconfirm
              title={"是否要删除" + record.area_name + "？"}
              onConfirm={() => { this.handleDelete(record) }}
              okText="确认"
              cancelText="取消"
            >
              <a href="#" style={{ color: "red" }}>删除</a>
            </Popconfirm>
          </Fragment> : ""
      )
    },
  ];

  /**
   * 加载完成事件处理
   */
  componentDidMount() {
    const { dispatch } = this.props;
    dispatch({
      type: 'area/fetch',
      payload: { id: '100000' }, // 默认 id 为 100000 获取全部省级区划信息
    });
  }

  /**
   * 分页、排序、筛选变化时触发
   */
  handleStandardTableChange = (
    pagination: Partial<TableListPagination>,
    filtersArg: Record<keyof TableListItem, string[]>,
    sorter: SorterResult<TableListItem>,
  ) => {
    const { dispatch } = this.props;
    const { formValues } = this.state;

    const filters = Object.keys(filtersArg).reduce((obj, key) => {
      const newObj = { ...obj };
      newObj[key] = getValue(filtersArg[key]);
      return newObj;
    }, {});

    const params: Partial<TableListParams> = {
      currentPage: pagination.current,
      pageSize: pagination.pageSize,
      ...formValues,
      ...filters,
    };
    if (sorter.field) {
      params.sorter = `${sorter.field}_${sorter.order}`;
    }
    // console.log('params===');
    // console.log(params.currentPage);
    dispatch({
      type: 'area/fetch',
      payload: params,
    });
  };

  // 删除
  handleDelete = (record: any) => {
    const { dispatch } = this.props;
    dispatch({
      type: 'area/remove',
      payload: { areaCode: record.area_code },
      callback: (data: any) => {
        if (data.success) {
          message.success("删除成功");
          // this.handleFormReset()
          dispatch({
            type: 'area/fetch',
            payload: { id: record.master_code },
          });
        } else {
          message.error("删除失败," + data.message)
        }
      }
    });
  }
  handleFormReset = () => {
    const { form, dispatch } = this.props;
    form.resetFields();
    this.setState({
      formValues: {},
    });
    dispatch({
      type: 'area/fetch',
      payload: {},
    });
  };

  toggleForm = () => {
    const { expandForm } = this.state;
    this.setState({
      expandForm: !expandForm,
    });
  };

  handleMenuClick = (e: { key: string }) => {
    const { dispatch } = this.props;
    const { selectedRows } = this.state;

    if (!selectedRows) return;
    switch (e.key) {
      case 'remove':
        dispatch({
          type: 'area/remove',
          payload: {
            key: selectedRows.map(row => row.key),
          },
          callback: () => {
            this.setState({
              selectedRows: [],
            });
          },
        });
        break;
      default:
        break;
    }
  };

  handleSelectRows = (rows: TableListItem[]) => {
    this.setState({
      selectedRows: rows,
    });
  };

  /**
   * 点击展开事件处理
   */
  handleExpand = (expanded: boolean, record: TableListItem) => {
    const { dispatch } = this.props;
    if (!expanded) return; // 如果是关闭就返回
    if (record.children && record.children.length > 0) return; // 如果已经有数据就返回
    dispatch({
      type: 'area/fetch',
      payload: {
        id: record.area_code,
      },
    });
  };

  handleSearch = (e: React.FormEvent) => {
    e.preventDefault();

    const { dispatch, form } = this.props;

    form.validateFields((err, fieldsValue) => {
      if (err) return;

      const values = {
        ...fieldsValue,
        updatedAt: fieldsValue.updatedAt && fieldsValue.updatedAt.valueOf(),
      };

      this.setState({
        formValues: values,
      });

      dispatch({
        type: 'area/fetch',
        payload: values,
      });
    });
  };

  handleModalVisible = (flag?: boolean) => {
    this.setState({
      modalVisible: !!flag,
    });
  };

  handleUpdateModalVisible = (flag?: boolean, record?: FormValueType) => {
    this.setState({
      updateModalVisible: !!flag,
      stepFormValues: record || {},
    });
  };

  // 添加行政区
  handleAdd = (fields: { desc: any }) => {
    const { dispatch } = this.props;
    dispatch({
      type: 'area/add',
      payload: {
        areaCode: fields.area[1].slice(0, 4) + fields.code,
        areaName: fields.name,
        masterCode: fields.area[1],
        areaTypeId: fields.type,
      },
      callback: (response: any) => {
        if (response.success) {
          message.success('添加成功');
          this.setState({ modalVisible: false });
          dispatch({
            type: 'area/fetch',
            payload: { id: '100000' }, // 默认 id 为 100000 获取全部省级区划信息
          });
        } else {
          message.error('添加失败,' + response.message);
        }
      },
    })
  };

  // 修改选择新建行政区父级
  changeCityCode = (e: string) => {
    this.setState({
      cityCode: e
    })
  }

  handleUpdate = (fields: FormValueType) => {
    const { dispatch } = this.props;
    dispatch({
      type: 'area/update',
      payload: {
        areaCode: fields.area[1].slice(0, 4) + fields.code,
        areaName: fields.name,
        masterCode: fields.area[1],
        areaTypeId: fields.type,
      },
      callback: (response: any) => {
        if (response.success) {
          message.success('修改成功');
          this.setState({ modalVisible: false });
          dispatch({
            type: 'area/fetch',
            payload: { id: '100000' }, // 默认 id 为 100000 获取全部省级区划信息
          });
        } else {
          message.error('修改失败,' + response.message);
        }
      },
    });
  };

  renderSimpleForm() {
    const { form } = this.props;
    const { getFieldDecorator } = form;
    return (
      <Form onSubmit={this.handleSearch} layout="inline">
        <Row gutter={{ md: 8, lg: 24, xl: 48 }}>
          <Col md={8} sm={24}>
            <FormItem label="规则名称">
              {getFieldDecorator('name')(<Input placeholder="请输入" />)}
            </FormItem>
          </Col>
          <Col md={8} sm={24}>
            <FormItem label="使用状态">
              {getFieldDecorator('status')(
                <Select placeholder="请选择" style={{ width: '100%' }}>
                  <Option value="0">关闭</Option>
                  <Option value="1">运行中</Option>
                </Select>,
              )}
            </FormItem>
          </Col>
          <Col md={8} sm={24}>
            <span className={styles.submitButtons}>
              <Button type="primary" htmlType="submit">
                查询
              </Button>
              <Button style={{ marginLeft: 8 }} onClick={this.handleFormReset}>
                重置
              </Button>
              <a style={{ marginLeft: 8 }} onClick={this.toggleForm}>
                展开 <Icon type="down" />
              </a>
            </span>
          </Col>
        </Row>
      </Form>
    );
  }

  renderAdvancedForm() {
    const {
      form: { getFieldDecorator },
    } = this.props;
    return (
      <Form onSubmit={this.handleSearch} layout="inline">
        <Row gutter={{ md: 8, lg: 24, xl: 48 }}>
          <Col md={8} sm={24}>
            <FormItem label="规则名称">
              {getFieldDecorator('name')(<Input placeholder="请输入" />)}
            </FormItem>
          </Col>
          <Col md={8} sm={24}>
            <FormItem label="使用状态">
              {getFieldDecorator('status')(
                <Select placeholder="请选择" style={{ width: '100%' }}>
                  <Option value="0">关闭</Option>
                  <Option value="1">运行中</Option>
                </Select>,
              )}
            </FormItem>
          </Col>
          <Col md={8} sm={24}>
            <FormItem label="调用次数">
              {getFieldDecorator('number')(<InputNumber style={{ width: '100%' }} />)}
            </FormItem>
          </Col>
        </Row>
        <Row gutter={{ md: 8, lg: 24, xl: 48 }}>
          <Col md={8} sm={24}>
            <FormItem label="更新日期">
              {getFieldDecorator('date')(
                <DatePicker style={{ width: '100%' }} placeholder="请输入更新日期" />,
              )}
            </FormItem>
          </Col>
          <Col md={8} sm={24}>
            <FormItem label="使用状态">
              {getFieldDecorator('status3')(
                <Select placeholder="请选择" style={{ width: '100%' }}>
                  <Option value="0">关闭</Option>
                  <Option value="1">运行中</Option>
                </Select>,
              )}
            </FormItem>
          </Col>
          <Col md={8} sm={24}>
            <FormItem label="使用状态">
              {getFieldDecorator('status4')(
                <Select placeholder="请选择" style={{ width: '100%' }}>
                  <Option value="0">关闭</Option>
                  <Option value="1">运行中</Option>
                </Select>,
              )}
            </FormItem>
          </Col>
        </Row>
        <div style={{ overflow: 'hidden' }}>
          <div style={{ float: 'right', marginBottom: 24 }}>
            <Button type="primary" htmlType="submit">
              查询
            </Button>
            <Button style={{ marginLeft: 8 }} onClick={this.handleFormReset}>
              重置
            </Button>
            <a style={{ marginLeft: 8 }} onClick={this.toggleForm}>
              收起 <Icon type="up" />
            </a>
          </div>
        </div>
      </Form>
    );
  }

  renderForm() {
    const { expandForm } = this.state;
    return expandForm ? this.renderAdvancedForm() : this.renderSimpleForm();
  }

  render() {
    const {
      area: { data },
      loading,
    } = this.props;

    // const data = this.state.data ? this.state.data.list : '';

    const { selectedRows, modalVisible, updateModalVisible, stepFormValues } = this.state;
    // const menu = (
    //   <Menu onClick={this.handleMenuClick} selectedKeys={[]}>
    //     <Menu.Item key="remove">删除</Menu.Item>
    //     <Menu.Item key="approval">批量审批</Menu.Item>
    //   </Menu>
    // );


    // 新建行政区传递
    const parentMethods = {
      handleAdd: this.handleAdd,
      handleModalVisible: this.handleModalVisible,
      cityCode: this.state.cityCode,
      changeCityCode: this.changeCityCode
    };


    const updateMethods = {
      handleUpdateModalVisible: this.handleUpdateModalVisible,
      handleUpdate: this.handleUpdate,
      changeCityCode: this.changeCityCode
    };

    // console.log(data, 'response props');
    // console.log(this.state, 'response state 1');
    return (
      <PageHeaderWrapper>
        <Card bordered={false}>
          <div className={styles.tableList}>
            {/* <div className={styles.tableListForm}>{this.renderForm()}</div> */}
            <div className={styles.tableListOperator}>
              <Button icon="plus" type="primary" onClick={() => this.handleModalVisible(true)}>
                新建
              </Button>
              {/* {selectedRows.length > 0 && (
                <span>
                  <Button>批量操作</Button>
                  <Dropdown overlay={menu}>
                    <Button>
                      更多操作 <Icon type="down" />
                    </Button>
                  </Dropdown>
                </span>
              )} */}
            </div>
            <StandardTable
              rowKey={record => record.area_code}
              rowSelection={false}
              selectedRows={selectedRows}
              loading={loading}
              defaultExpandedRowKeys={[6]}
              onExpand={this.handleExpand}
              data={data}
              columns={this.columns}
              onSelectRow={this.handleSelectRows}
              pagination={false}
              onChange={this.handleStandardTableChange}
            // indentSize={50}
            />
          </div>
        </Card>
        {/* 新建行政区 */}
        <CreateForm {...parentMethods} modalVisible={modalVisible} />


        {stepFormValues && Object.keys(stepFormValues).length ? (
          <EditForm
            {...updateMethods}
            updateModalVisible={updateModalVisible}
            values={stepFormValues}
          />
        ) : null}
      </PageHeaderWrapper>
    );
  }
}

export default Form.create<TableListProps>()(TableList);
