import { Axis, Chart, Geom, Tooltip } from 'bizcharts';
import { Select } from 'antd';
import React, { Component } from 'react';
import Slider from 'bizcharts-plugin-slider';
import DataSet from '@antv/data-set';
import Debounce from 'lodash.debounce';
import autoHeight from '../autoHeight';
import styles from '../index.less';

const { Option } = Select;

export interface BarProps {
  title: React.ReactNode;
  color?: string;
  padding?: [number, number, number, number];
  height?: number;
  data: {
    x: string;
    y: number;
  }[];
  forceFit?: boolean;
  autoLabel?: boolean;
  style?: React.CSSProperties;
  props: {};
  keyType: string;
  code: string;
}

const ds = new DataSet({
  state: {
    start: 0,
    end: 0.08,
  },
});

class Bar extends Component<
  BarProps,
  {
    autoHideXLabels: boolean;
    learningSection: [];
    // codeNum: string;
  }
  > {
  state = {
    autoHideXLabels: false,
    learningSection: [],
    // codeNum: "150430"
  };

  root: HTMLDivElement | undefined = undefined;

  node: HTMLDivElement | undefined = undefined;

  resize = Debounce(() => {
    if (!this.node || !this.node.parentNode) {
      return;
    }
    const canvasWidth = (this.node.parentNode as HTMLDivElement).clientWidth;
    const { data = [], autoLabel = true } = this.props;
    if (!autoLabel) {
      return;
    }
    const minWidth = data.length * 30;
    const { autoHideXLabels } = this.state;

    if (canvasWidth <= minWidth) {
      if (!autoHideXLabels) {
        this.setState({
          autoHideXLabels: true,
        });
      }
    } else if (autoHideXLabels) {
      this.setState({
        autoHideXLabels: false,
      });
    }
  }, 500);

  componentDidMount() {
    window.addEventListener('resize', this.resize, { passive: true });

    const { dispatch } = this.props.props;
    dispatch({
      type: 'dashboardAnalysis/stageInfo',
      callback: (data: any) => {
        this.setState({ learningSection: data.list })
      }
    });
  }

  componentWillUnmount() {
    window.removeEventListener('resize', this.resize);
  }

  handleRoot = (n: HTMLDivElement) => {
    this.root = n;
  };

  handleRef = (n: HTMLDivElement) => {
    this.node = n;
  };

  handleSliderChange = (e: any) => {
    const { startRadio, endRadio } = e;
    ds.setState('start', startRadio);
    ds.setState('end', endRadio);
  };
  handleChange = (value: string) => {
    // console.log(this.props, "123456465")
    const { dispatch } = this.props.props;
    if (this.props.keyType === "student") {
      dispatch({
        type: 'dashboardAnalysis/schoolStudentNum',
        payload: {
          districtCode: this.props.code,
          xxbxlx: value
        }
      });
    } else {
      dispatch({
        type: 'dashboardAnalysis/schoolTeacherNum',
        payload: {
          districtCode: this.props.code,
          xxbxlx: value
        }
      });
    }

  }

  render() {
    const {
      height = 1,
      title,
      forceFit = true,
      data,
      color = 'rgba(24, 144, 255, 0.85)',
      padding,
    } = this.props;

    const { autoHideXLabels } = this.state;

    // const scale = {
    //   count: {
    //     min: 0
    //   }

    // };
    var scale = {};
    if (data.length > 0) {
      var a = Math.max.apply(Math, data.map((item) => { return item.count }))
      if (a <= 2 && a > 0) {
        scale = {
          count: {
            min: 0,
            range: [
              0,
              1
            ],
            ticks: [
              0,
              1,
              2,
            ]
          }
        };
      } else {
        scale = {
          count: {
            min: 0,
            // tickCount: 5,
          }
        };
      }
    }

    const tooltip: [string, (...args: any[]) => { name?: string; value: string }] = [
      'org_name*count',
      (x: string, y: string) => ({
        name: x,
        value: y,
      }),
    ];

    const dv = ds.createView({ origin }).source(data);
    // console.log(ds)
    dv.transform({
      type: 'filter',
      callback(item: any, idx: any) {
        const radio = idx / data.length;
        return radio >= ds.state.start && radio <= ds.state.end;
      },
    });
    return (
      [this.props.keyType === "student" || this.props.keyType === "teacher" ? <div>
        学校办学类型：
          <Select defaultValue="-1" style={{ width: 120 }} onChange={this.handleChange}>
          <Option value="-1">全部</Option>
          {this.state.learningSection.map((item: any) => {
            return <Option value={item.dict_code}>{item.dict_value}</Option>
          })}
        </Select>
      </div> : "",
      <div className={styles.chart} style={{ height }} ref={this.handleRoot}>
        <div ref={this.handleRef}>
          {title && <h4 style={{ marginBottom: 20 }}>{title}</h4>}
          <Chart
            scale={scale}
            height={title ? height - 41 : height}
            forceFit={forceFit}
            data={dv}
            padding={padding || 'auto'}
            onClick={(views: any) => {
              if (views) {
                const { _origin } = views.data;
                // console.log(_origin,"55555555555")
                // console.log(views)
                window.location.href = `#/dashboard/workplace/analysis/monitor/${_origin.org_id}?org_name=` + _origin.org_name;
              }
            }}
          >
            <Axis
              name="org_name"
              title={false}
              label={autoHideXLabels ? undefined : {}}
              tickLine={autoHideXLabels ? undefined : {}}
            />
            <Axis name="count" min={0} />
            <Tooltip showTitle={false} crosshairs={false} />
            <Geom type="interval" position="org_name*count" color={color} tooltip={tooltip} style={{ cursor: "pointer" }} />
          </Chart>
          {data.length > 0 ? <Slider
            start={ds.state.start}
            end={ds.state.end * data.length}
            data={data}
            padding={60}
            xAxis="org_name"
            yAxis="count"
            onChange={this.handleSliderChange}
          />
            : ""}

        </div>
      </div>]
    );
  }
}

export default autoHeight()(Bar);
